export class SkipList {
    comparator;
    maxLevel;
    random;
    root;
    level = 0;
    count = 0;
    constructor(comparator, maxLevel = 8, random = () => Math.random()) {
        this.comparator = comparator;
        this.maxLevel = maxLevel;
        this.random = random;
        this.root = this.buildNode(null, this.maxLevel);
    }
    buildUpdate(value) {
        const update = Array(this.maxLevel + 1);
        let current = this.root;
        for (let i = this.level; i >= 0; i--) {
            let nextForward = current?.forward[i];
            while (nextForward != null && this.comparator(nextForward.value, value) < 0) {
                current = nextForward;
                nextForward = nextForward.forward[i];
            }
            update[i] = current;
        }
        current = current?.forward[0];
        return { current, update };
    }
    add(value) {
        let { current, update } = this.buildUpdate(value);
        if (current == null || this.comparator(current.value, value) !== 0) {
            const level = this.randomLevel();
            if (level > this.level) {
                for (let i = this.level + 1; i <= level; i++) {
                    update[i] = this.root;
                }
                this.level = level;
            }
            const node = this.buildNode(value, level);
            for (let i = 0; i <= level; i++) {
                const toUpdate = update[i];
                if (toUpdate != null) {
                    node.forward[i] = toUpdate.forward[i];
                    toUpdate.forward[i] = node;
                }
            }
            this.count++;
        }
    }
    remove(value) {
        let { current, update } = this.buildUpdate(value);
        if (current != null && this.comparator(current.value, value) === 0) {
            for (let i = 0; i <= this.level; i++) {
                const toUpdate = update[i];
                if (toUpdate?.forward[i] !== current) {
                    break;
                }
                if (toUpdate != null) {
                    toUpdate.forward[i] = current.forward[i];
                }
            }
            while (this.level > 0 && this.root.forward[this.level] == null) {
                this.level--;
            }
            this.count--;
        }
    }
    get length() {
        return this.count;
    }
    peek() {
        return this.root.forward[0]?.value;
    }
    take() {
        if (this.count === 0) {
            return undefined;
        }
        const headValue = this.peek();
        this.remove(headValue);
        return headValue;
    }
    toArray() {
        return Array.from(this.values());
    }
    *values() {
        for (const [value] of this.valuesAndIndexes()) {
            yield value;
        }
    }
    *valuesAndIndexes() {
        let index = 0;
        let current = this.root.forward[0];
        while (current != null) {
            yield [current.value, index];
            index++;
            current = current.forward[0];
        }
    }
    buildNode(value, level) {
        return { forward: Array(level + 1), value };
    }
    randomLevel() {
        return Math.trunc(this.random() * this.maxLevel + 1);
    }
}
export const skipList = (comparator) => new SkipList(comparator);
//# sourceMappingURL=skip-list.js.map